@@ -4,7 +4,6 @@ use std::{
     path::{Path, PathBuf},
     rc::Rc,
     sync::Arc,
-    time::Instant,
 };
 
 use exocore_chain::{
@@ -14,7 +13,7 @@ use exocore_chain::{
     operation::{Operation, OperationId},
     pending, EngineHandle, EngineOperationStatus,
 };
-use exocore_core::{cell::FullCell, time::Clock};
+use exocore_core::{cell::FullCell, time::Clock, time::Instant};
 use exocore_protos::{
     generated::exocore_store::{
         entity_mutation::Mutation, Entity, EntityMutation, EntityQuery,
@@ -94,11 +93,12 @@ where
         pending_index.set_full_text_boost(config.pending_index_boost);
 
         // make sure directories are created
-        let mut chain_index_dir = cell
+        let chain_index_dir = cell
             .cell()
             .store_directory()
-            .ok_or_else(|| anyhow!("Cell doesn't have an path configured"))?;
-        chain_index_dir.push("chain");
+            .as_os_path()
+            .expect("Expected cell to be in an OS directory")
+            .join("chain");
         if std::fs::metadata(&chain_index_dir).is_err() {
             std::fs::create_dir_all(&chain_index_dir)?;
         }
